# this script has been tested on Windows 10 using the mdf2parquet.exe v2.3.2
# update the script with your container names and connection strings to run it
# and make sure to place the MF4 decoder and prefixed DBC files next to the script

# WARNING: The script is deliberately light and includes no error handling.
# Use it at your own risk. Also, consider a step-by-step approach for large data volumes

from google.cloud import storage
from google.oauth2 import service_account
import os, subprocess, sys

# Specify the below variables 
bucket_input_name = "your-input-bucket"
devices_to_process = [] # keep blank to process all or specify as e.g. ["7512BE4D", "143ABFF8"]

# --------------------------------------------
# Output bucket based on input bucket name 
bucket_output_name = bucket_input_name + "-parquet" 

# Path to decoder and input/output folders
decoder_path = "mdf2parquet_decode.exe"
fs_logfile_path = "input"
fs_parquet_path = "output"

# check if decoder and dbc files are available
script_dir = os.path.dirname(os.path.abspath(__file__))

if not os.path.isfile(decoder_path):
    print(f"Warning: {decoder_path} is missing. Exiting.")
    sys.exit()
    
dbc_files = [file for file in os.listdir(script_dir) if file.endswith('.dbc') and (file.startswith('can') or file.startswith('lin'))]
if not dbc_files:
    print("Warning: No DBC files found with valid prefixes, e.g. 'can1' or 'lin2'. Exiting.")
    sys.exit()
    
# Path to your service account key file
key_path = "bigquery-storage-admin-account.json"

# Explicitly use service account credentials by specifying the private key file.
credentials = service_account.Credentials.from_service_account_file(key_path)

# Initialize the client with explicit credentials
storage_client = storage.Client(credentials=credentials)

# Get GCS buckets
bucket_input = storage_client.get_bucket(bucket_input_name)
bucket_output = storage_client.get_bucket(bucket_output_name)

os.makedirs(fs_logfile_path, exist_ok=True)
os.makedirs(fs_parquet_path, exist_ok=True)

# List objects in the input bucket and filter by prefix
blobs = bucket_input.list_blobs(prefix="")  # Adjust prefix as needed

deviceids = set(blob.name.split("/")[0] for blob in blobs if "/" in blob.name)

# Valid file extensions
valid_extensions = {".MF4", ".MFC", ".MFE", ".MFM"}
downloaded_files = 0

for deviceid in deviceids:
    print(f"\nProcessing device ID {deviceid}\n")
    if deviceid not in devices_to_process and len(devices_to_process) != 0:
        print(f"Device ID {deviceid} is not part of devices to process")
        continue 
    
    # Create directory for device ID
    device_dir = os.path.join(fs_logfile_path, deviceid)
    os.makedirs(device_dir, exist_ok=True)

    # Handle session folders under each device ID
    session_blobs = bucket_input.list_blobs(prefix=f"{deviceid}/")
    session_folders = set(
        blob.name.split("/")[1]
        for blob in session_blobs
        if len(blob.name.split("/")) > 1
    )

    for session in session_folders:
        session_dir = os.path.join(device_dir, session)
        os.makedirs(session_dir, exist_ok=True)

        # Filter blobs by device ID, session folder, and check extension
        session_blobs = bucket_input.list_blobs(prefix=f"{deviceid}/{session}/")
        for blob in session_blobs:
            if os.path.splitext(blob.name)[1].upper() in valid_extensions:
                download_path = os.path.join(session_dir, os.path.basename(blob.name))
                blob.download_to_filename(download_path)
                downloaded_files += 1

print(f"Completed downloading {downloaded_files} MDF files")

# -----------
# Process downloaded files

result = subprocess.run(
    [
        str(decoder_path),
        "-i",
        str(fs_logfile_path),
        "-O",
        str(fs_parquet_path),
        "--verbosity=5",
        "-X",
    ],
    text=True,
    capture_output=True,
)

print(f"\nProcessed downloaded MDF log files via the mdf2parquet decoder:")
print(f"{result.stdout}")
print(f"{result.stderr}")

# -----------
# Upload resulting Parquet files
uploaded_files = 0
for root, dirs, files in os.walk(fs_parquet_path):
    for file in files:
        if file.endswith(".parquet"):
            file_path = os.path.join(root, file)
            blob_path = os.path.relpath(file_path, fs_parquet_path).replace("\\","/")
            blob = bucket_output.blob(blob_path)
            with open(file_path, "rb") as data:
                blob.upload_from_file(data)
                uploaded_files += 1

print(f"Completed uploading {uploaded_files} Parquet files to {bucket_output_name}.")
